open import Lib.Equality as Eq using (_==_ ; auto)
open import Lib.Nat
open import Lib.Maybe
open import Lib.Decidable
open import Lib.Boolean
open import Lib.Proposition
open import Lib.List
open import Lib.Word
open import Lib.Word-Lemmas
open import Lib.Word-Tactics

open Monoid-Equational
open Rewriting
open Associative
open InContext

open import Lib.MaybeEq
open import Lib.MaybeEq-Instances
open import CosetNF
open import Theorem
open CliffordCS

module Step8.Comm where

  -- Commutativity.
  comm : (x y : Gen) -> Maybe (commutes Rel x y)

  comm S0-gen S1-gen = Just (symm (axiom ax-S1-S0=S0-S1))
  comm S0-gen S2-gen = Just (symm (axiom ax-S2-S0=S0-S2))
  comm S0-gen CS01-gen = Just (symm (axiom ax-CS01-S0=S0-CS01))
  comm S0-gen CS12-gen = Just (symm (axiom ax-CS12-S0=S0-CS12))
  comm S0-gen iI-gen = Just (symm (axiom ax-iI-S0=S0-iI))
  comm S0-gen K1-gen = Just (symm (axiom ax-K1-S0=S0-K1))
  comm S0-gen K2-gen = Just (symm (axiom ax-K2-S0=S0-K2))
  comm S1-gen S0-gen = Just (axiom ax-S1-S0=S0-S1)
  comm S1-gen S2-gen = Just (symm (axiom ax-S2-S1=S1-S2))
  comm S1-gen CS01-gen = Just (symm (axiom ax-CS01-S1=S1-CS01))
  comm S1-gen CS12-gen = Just (symm (axiom ax-CS12-S1=S1-CS12))
  comm S1-gen iI-gen = Just (symm (axiom ax-iI-S1=S1-iI))
  comm S1-gen K0-gen = Just (axiom ax-S1-K0=K0-S1)
  comm S1-gen K2-gen = Just (symm (axiom ax-K2-S1=S1-K2))
  comm S2-gen S0-gen = Just (axiom ax-S2-S0=S0-S2)
  comm S2-gen S1-gen = Just (axiom ax-S2-S1=S1-S2)
  comm S2-gen CS01-gen = Just (symm (axiom ax-CS01-S2=S2-CS01))
  comm S2-gen CS12-gen = Just (symm (axiom ax-CS12-S2=S2-CS12))
  comm S2-gen iI-gen = Just (symm (axiom ax-iI-S2=S2-iI))
  comm S2-gen K0-gen = Just (axiom ax-S2-K0=K0-S2)
  comm S2-gen K1-gen = Just (axiom ax-S2-K1=K1-S2)
  comm CS01-gen S0-gen = Just (axiom ax-CS01-S0=S0-CS01)
  comm CS01-gen S1-gen = Just (axiom ax-CS01-S1=S1-CS01)
  comm CS01-gen S2-gen = Just (axiom ax-CS01-S2=S2-CS01)
  comm CS01-gen CS12-gen = Just (symm (axiom ax-CS12-CS01=CS01-CS12))
  comm CS01-gen iI-gen = Just (symm (axiom ax-iI-CS01=CS01-iI))
  comm CS01-gen K2-gen = Just (axiom ax-CS01-K2=K2-CS01)
  comm CS12-gen S0-gen = Just (axiom ax-CS12-S0=S0-CS12)
  comm CS12-gen S1-gen = Just (axiom ax-CS12-S1=S1-CS12)
  comm CS12-gen S2-gen = Just (axiom ax-CS12-S2=S2-CS12)
  comm CS12-gen CS01-gen = Just (axiom ax-CS12-CS01=CS01-CS12)
  comm CS12-gen CS12-gen = Just refl
  comm CS12-gen iI-gen = Just (symm (axiom ax-iI-CS12=CS12-iI))
  comm CS12-gen K0-gen = Just (axiom ax-CS12-K0=K0-CS12)
  comm iI-gen S0-gen = Just (axiom ax-iI-S0=S0-iI)
  comm iI-gen S1-gen = Just (axiom ax-iI-S1=S1-iI)
  comm iI-gen S2-gen = Just (axiom ax-iI-S2=S2-iI)
  comm iI-gen CS01-gen = Just (axiom ax-iI-CS01=CS01-iI)
  comm iI-gen CS12-gen = Just (axiom ax-iI-CS12=CS12-iI)
  comm iI-gen K0-gen = Just (axiom ax-iI-K0=K0-iI)
  comm iI-gen K1-gen = Just (axiom ax-iI-K1=K1-iI)
  comm iI-gen K2-gen = Just (axiom ax-iI-K2=K2-iI)
  comm K0-gen S1-gen = Just (symm (axiom ax-S1-K0=K0-S1))
  comm K0-gen S2-gen = Just (symm (axiom ax-S2-K0=K0-S2))
  comm K0-gen CS12-gen = Just (symm (axiom ax-CS12-K0=K0-CS12))
  comm K0-gen iI-gen = Just (symm (axiom ax-iI-K0=K0-iI))
  comm K0-gen K1-gen = Just (symm (axiom ax-K1-K0=K0-K1))
  comm K0-gen K2-gen = Just (symm (axiom ax-K2-K0=K0-K2))
  comm K1-gen S0-gen = Just (axiom ax-K1-S0=S0-K1)
  comm K1-gen S2-gen = Just (symm (axiom ax-S2-K1=K1-S2))
  comm K1-gen iI-gen = Just (symm (axiom ax-iI-K1=K1-iI))
  comm K1-gen K0-gen = Just (axiom ax-K1-K0=K0-K1)
  comm K1-gen K2-gen = Just (symm (axiom ax-K2-K1=K1-K2))
  comm K2-gen S0-gen = Just (axiom ax-K2-S0=S0-K2)
  comm K2-gen S1-gen = Just (axiom ax-K2-S1=S1-K2)
  comm K2-gen CS01-gen = Just (symm (axiom ax-CS01-K2=K2-CS01))
  comm K2-gen iI-gen = Just (symm (axiom ax-iI-K2=K2-iI))
  comm K2-gen K0-gen = Just (axiom ax-K2-K0=K0-K2)
  comm K2-gen K1-gen = Just (axiom ax-K2-K1=K1-K2)
  comm x y = Nothing

  -- We number the generators for the purpose of ordering them.
  ord : Gen -> Nat
  ord S0-gen = 0
  ord S1-gen = 1
  ord S2-gen = 2
  ord CS01-gen = 3
  ord CS12-gen = 4
  ord iI-gen = 8
  ord K0-gen = 5
  ord K1-gen = 6
  ord K2-gen = 7
  
  -- Ordering of generators.
  less : Gen -> Gen -> Boolean
  less x y with ord x <=? ord y
  less x y | yes _ = true
  less x y | no _ = false
  
  open Commuting Gen Rel comm less public

  nf-comm : ListNF Rel
  nf-comm = record { listnf = comm-canonical ; lemma-listnf = lemma-comm-canonical }
